import logging
import pandas as pd

from django.core.management.base import BaseCommand
from django.utils import timezone

from openlxp_xia.management.utils.xia_internal import (
    is_date, required_recommended_logs)
from openlxp_xia.management.utils.xss_client import (
    get_data_types_for_validation, get_required_fields_for_validation,
    get_target_validation_schema)
from openlxp_xia.models import (MetadataLedger, SupplementalLedger,
                                XIAConfiguration)

logger = logging.getLogger('dict_config_logger')


def get_target_metadata_for_validation():
    """Retrieving target metadata from MetadataLedger that needs to be
        validated"""
    logger.info(
        "Accessing target metadata from MetadataLedger to be validated")
    target_data_dict = MetadataLedger.objects.values(
        'target_metadata_key_hash',
        'target_metadata').filter(target_metadata_validation_status='',
                                  record_lifecycle_status='Active',
                                  #   target_metadata_transmission_date=None
                                  ).exclude(
        source_metadata_transformation_date=None)
    return target_data_dict


def update_previous_instance_in_metadata(key_value_hash):
    """Update older instances of record to inactive status"""
    # Setting record_status & deleted_date for updated record
    MetadataLedger.objects.filter(
        source_metadata_key_hash=key_value_hash,
        record_lifecycle_status='Active'). \
        exclude(target_metadata_validation_date=None).update(
        metadata_record_inactivation_date=timezone.now())
    MetadataLedger.objects.filter(
        source_metadata_key_hash=key_value_hash,
        record_lifecycle_status='Active'). \
        exclude(target_metadata_validation_date=None).update(
        record_lifecycle_status='Inactive')

    SupplementalLedger.objects.filter(
        supplemental_metadata_key_hash=key_value_hash,
        record_lifecycle_status='Active'). \
        exclude(supplemental_metadata_validation_date=None).update(
        metadata_record_inactivation_date=timezone.now())
    SupplementalLedger.objects.filter(
        supplemental_metadata_key_hash=key_value_hash,
        record_lifecycle_status='Active'). \
        exclude(supplemental_metadata_validation_date=None).update(
        record_lifecycle_status='Inactive')


def store_target_metadata_validation_status(target_data_dict, key_value_hash,
                                            validation_result,
                                            record_status_result,
                                            target_metadata):
    """Storing validation result in MetadataLedger"""
    if record_status_result == 'Active':
        update_previous_instance_in_metadata(key_value_hash)
        target_data_dict.filter(
            target_metadata_key_hash=key_value_hash).update(
            target_metadata=target_metadata,
            target_metadata_validation_status=validation_result,
            target_metadata_validation_date=timezone.now(),
            record_lifecycle_status=record_status_result)

    else:
        target_data_dict.filter(
            target_metadata_key_hash=key_value_hash).update(
            target_metadata=target_metadata,
            target_metadata_validation_status=validation_result,
            target_metadata_validation_date=timezone.now(),
            record_lifecycle_status=record_status_result,
            metadata_record_inactivation_date=timezone.now())

    SupplementalLedger.objects.filter(
        supplemental_metadata_key_hash=key_value_hash,
        record_lifecycle_status="Active").update(
        supplemental_metadata_validation_date=timezone.now(),
        record_lifecycle_status=record_status_result)


def validate_target_using_key(target_data_dict, required_column_list,
                              recommended_column_list, expected_data_types):
    """Validating target data against required & recommended column names"""

    logger.info('Validating and updating records in MetadataLedger table for '
                'target data')
    if target_data_dict:
        target_metadata = (target_data_dict.values_list(
            'target_metadata', flat=True))
        len_target_metadata = len(target_metadata)
        flattened_df = pd.json_normalize(target_metadata)
        flattened_dict = flattened_df.to_dict(orient='index')
        for ind in range(len_target_metadata):
            # Updating default validation for all records
            validation_result = 'Y'
            record_status_result = 'Active'

            flattened_source_data = flattened_dict[ind]

            # validate for required values in data
            for item_name in required_column_list:
                # update validation and record status for invalid data
                # Log out error for missing required values
                # item_name = item[:-len(".use")]
                if item_name in flattened_source_data:
                    if not flattened_source_data[item_name]:
                        validation_result = 'N'
                        record_status_result = 'Inactive'
                        required_recommended_logs(ind, "Required", item_name)
                else:
                    validation_result = 'N'
                    record_status_result = 'Inactive'
                    required_recommended_logs(ind, "Required", item_name)

            # validate for recommended values in data
            for item_name in recommended_column_list:
                # Log out warning for missing recommended values
                # item_name = item[:-len(".use")]
                if item_name in flattened_source_data:
                    if not flattened_source_data[item_name]:
                        required_recommended_logs(ind, "Recommended",
                                                  item_name)
                else:
                    required_recommended_logs(ind, "Recommended", item_name)
            # Type checking for values in metadata
            for item in flattened_source_data:
                # check if datatype has been assigned to field
                if item in expected_data_types:
                    # type checking for datetime datatype fields
                    if expected_data_types[item] == "datetime":
                        if not is_date(flattened_source_data[item]):
                            required_recommended_logs(ind, "datatype",
                                                      item)
                    # type checking for datatype fields(except datetime)
                    elif (not isinstance(flattened_source_data[item],
                                         expected_data_types[item])):
                        required_recommended_logs(ind, "datatype",
                                                  item)

            # assigning key hash value for source metadata
            key_value_hash = target_data_dict[ind]['target_metadata_key_hash']
            # Calling function to update validation status
            store_target_metadata_validation_status(target_data_dict,
                                                    key_value_hash,
                                                    validation_result,
                                                    record_status_result,
                                                    target_data_dict[ind]
                                                    ['target_metadata'])


class Command(BaseCommand):
    """Django command to validate target data"""

    def add_arguments(self, parser):
        parser.add_argument('--config_id', type=int, help='ID of the config')

    def handle(self, *args, **options):
        """
            target data is validated and stored in metadataLedger
        """
        xia = None
        # Check if xia configuration is provided in options
        if 'config' in options:
            xia = options['config'].xia_configuration
            logger.info(xia)
        elif 'config_id' in options:
            # If config_id is provided, fetch the XIAConfiguration object
            try:
                xia = XIAConfiguration.objects.get(id=options['config_id'])
                logger.info(xia)
            except XIAConfiguration.DoesNotExist:
                logger.error(
                    f'XIA Configuration with ID {options["config_id"]}'
                    'does not exist')
                return
        if not xia:
            # If xia is not provided, log an error and exit
            xia = XIAConfiguration.objects.first()
            if not xia:
                logger.error('XIA Configuration is not provided')
                raise SystemExit('XIA Configuration is not provided')
        schema_data_dict = get_target_validation_schema(xia)
        target_data_dict = get_target_metadata_for_validation()
        required_column_list, recommended_column_list = \
            get_required_fields_for_validation(
                schema_data_dict)
        expected_data_types = get_data_types_for_validation(schema_data_dict)
        validate_target_using_key(target_data_dict, required_column_list,
                                  recommended_column_list, expected_data_types)
        logger.info(
            'MetadataLedger updated with target metadata validation status')



import json
import logging

import requests
from django.core.management.base import BaseCommand
from django.core.serializers.json import DjangoJSONEncoder
from django.db.models import Q
from django.utils import timezone

from openlxp_xia.management.utils.xia_internal import get_publisher_detail
from openlxp_xia.management.utils.xis_client import \
    posting_supplemental_metadata_to_xis
from openlxp_xia.models import SupplementalLedger

from openlxp_xia.models import (XISConfiguration,
                                supplementalTransmissionStatus as meta_status)

logger = logging.getLogger('dict_config_logger')


def rename_supplemental_metadata_fields(xis, data):
    """Renaming XIA column names to match with XIS column names"""
    data['unique_record_identifier'] = data.pop('metadata_record_uuid')
    data['metadata'] = data.pop('supplemental_metadata')
    data['metadata_hash'] = data.pop('supplemental_metadata_hash')
    data['metadata_key'] = data.pop('supplemental_metadata_key')
    data['metadata_key_hash'] = data.pop('supplemental_metadata_key_hash')
    # Adding Publisher in the list to POST to XIS
    data['provider_name'] = get_publisher_detail(xis)
    return data


def post_supplemental_metadata_to_xis(xis, data):
    """POSTing XIA metadata_ledger to XIS metadata_ledger"""
    # Traversing through each row one by one from data
    # get_xis_supplemental_metadata_api_endpoint
    for row in data:
        data = rename_supplemental_metadata_fields(xis, row)
        renamed_data = json.dumps(data, cls=DjangoJSONEncoder)

        # Getting UUID to update target_metadata_transmission_status to pending
        uuid_val = data.get('unique_record_identifier')

        metadata_record = SupplementalLedger.objects.filter(
            metadata_record_uuid=uuid_val).first()

        if metadata_record:

            trans_status = meta_status.objects.filter(
                metadata_record=metadata_record,
                XISConfiguration=xis
            ).first()

            trans_status.supplemental_metadata_transmission_status = 'Pending'

        # POSTing data to XIS
            try:
                xis_response = \
                    posting_supplemental_metadata_to_xis(xis,
                                                         renamed_data)

                # Receiving XIS response after validation and updating
                # metadata_ledger
                if xis_response.status_code == 201:
                    trans_status.target_metadata_transmission_status = \
                        'Successful'
                    trans_status.target_metadata_transmission_date = \
                        timezone.now()
                    trans_status.target_metadata_transmission_status_code = \
                        xis_response.status_code
                    logger.info(
                        "Data " + str(metadata_record) +
                        "successfully posted to XIS with status code "
                        + str(xis_response.status_code))
                else:
                    trans_status.target_metadata_transmission_status = 'Failed'
                    trans_status.target_metadata_transmission_date = \
                        timezone.now()
                    trans_status.target_metadata_transmission_status_code = \
                        xis_response.status_code
                    logger.warning(
                        "Bad request sent " + str(xis_response.status_code)
                        + "error found " + xis_response.text)
                trans_status.save()
            except requests.exceptions.RequestException as e:
                logger.error(e)
                # Updating status in XIA metadata_ledger to 'Failed'
                trans_status.target_metadata_transmission_status = 'Failed'
                trans_status.save()
                raise SystemExit('Exiting! Can not make connection with XIS.')

    load_supplemental_metadata_to_xis(xis)


def load_supplemental_metadata_to_xis(xis):
    """Retrieve number of Metadata_Ledger records in XIA to load into XIS  and
    calls the post_data_to_xis accordingly"""

    records = SupplementalLedger.objects.filter(
        record_lifecycle_status='Active').\
        exclude(supplemental_metadata_validation_date__isnull=True)

    if xis:
        for record in records:
            meta_status.objects.get_or_create(
                metadata_record=record,
                XISConfiguration=xis,
                defaults={'target_metadata_transmission_status': 'Ready'}
            )
        combined_query = meta_status.objects.filter(
            Q(target_metadata_transmission_status='Ready') | Q(
                target_metadata_transmission_status='Failed') | Q(
                target_metadata_transmission_status='Pending') &
            Q(XISConfiguration=xis))

        data = SupplementalLedger.objects.filter(
            meta_status__in=combined_query,
            record_lifecycle_status='Active'
        ).exclude(
            meta_status__target_metadata_transmission_status_code=400
        ).values(
            'metadata_record_uuid',
            'supplemental_metadata',
            'supplemental_metadata_hash',
            'supplemental_metadata_key',
            'supplemental_metadata_key_hash')

        # Checking available no. of records in XIA to load
        # into XIS is Zero or not
        if len(data) == 0:
            logger.info("Supplemental Metadata Loading in XIS is complete, "
                        "Zero records are available in XIA to transmit")
        else:
            post_supplemental_metadata_to_xis(xis, data)


class Command(BaseCommand):
    """Django command to load supplemental metadata in the Experience Index
    Service (XIS)"""

    def add_arguments(self, parser):
        parser.add_argument('--config_id', type=int, help='ID of the config')

    def handle(self, *args, **options):
        """Metadata is load from XIA Supplemental_Ledger to XIS
        Metadata_Ledger"""
        xis = None
        if 'config' in options:
            xis = options['config'].xis_configuration
            logger.info(xis)
        elif 'config_id' in options:
            # If config_id is provided, fetch the XIAConfiguration object
            try:
                xis = XISConfiguration.objects.get(id=options['config_id'])
                logger.info(xis)
            except XISConfiguration.DoesNotExist:
                logger.error(
                    f'XIS Configuration with ID {options["config_id"]}'
                    ' does not exist')
                return
        if not xis:
            # If xis is not provided, log an error and exit
            xis = XISConfiguration.objects.first()
            if not xis:
                logger.error('XIS Configuration is not provided')
                raise SystemExit('XIS Configuration is not provided')
        load_supplemental_metadata_to_xis(xis)



import html
import bleach
import logging
from confusable_homoglyphs import categories, confusables

logger = logging.getLogger('dict_config_logger')


def bleach_data_to_json(rdata):
    """Recursive function to bleach/clean HTML tags from string
    data and return dictionary data.

    :param rdata: dictionary to clean.
    WARNING rdata will be edited
    :return: dict"""

    keysList = list(rdata.keys())
    for key in keysList:
        if isinstance(rdata[key], str):
            # if string, clean
            rdata[key] = bleach.clean(rdata[key], tags={}, strip=True)
            rdata[key] = html.unescape(rdata[key])
        if isinstance(rdata[key], dict):
            # if dict, enter dict
            rdata[key] = bleach_data_to_json(rdata[key])
    return rdata


def is_safe_string(data):
    """Checks for dangerous homoglyphs."""

    return not (isinstance(data, str) and confusables.is_dangerous(data))


def confusable_homoglyphs_check(d, path=None):
    """
    Recursively iterate to every leaf node
    in a nested dictionary and apply check_func.
    Returns True if all leaf nodes pass the check, False otherwise.
    """
    if path is None:
        path = []
    result = True
    for k, v in d.items():
        if isinstance(v, dict):
            if not confusable_homoglyphs_check(v, path + [k]):
                logger.info(
                    "Homoglyphs does not have the expected preferred alias")
                logger.error(categories.unique_aliases(str(v)))
                result = False
        else:
            if not is_safe_string(v):
                logger.info(
                    "Homoglyphs does not have the expected preferred alias")
                logger.error(categories.unique_aliases(str(v)))
                result = False
    return result



# Generated by Django 3.2.18 on 2025-07-23 20:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openlxp_xia', '0009_auto_20250225_1536'),
    ]

    operations = [
        migrations.AlterField(
            model_name='metadatafieldoverwrite',
            name='field_type',
            field=models.CharField(choices=[('datetime', 'DATETIME'), ('int', 'INTEGER'), ('str', 'CHARACTER'), ('URI', 'CHARACTER'), ('bool', 'BOOLEAN')], max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name='xisconfiguration',
            name='xis_api_key',
            field=models.CharField(help_text='Enter the XIS API Key', max_length=512),
        ),
    ]



# Generated by Django 3.2.18 on 2025-08-04 20:15

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('openlxp_xia', '0013_supplementaltransmissionstatus'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='supplementalledger',
            name='supplemental_metadata_transmission_date',
        ),
        migrations.RemoveField(
            model_name='supplementalledger',
            name='supplemental_metadata_transmission_status',
        ),
        migrations.RemoveField(
            model_name='supplementalledger',
            name='supplemental_metadata_transmission_status_code',
        ),
    ]



# Generated by Django 3.2.18 on 2025-07-31 17:54

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    dependencies = [
        ('openlxp_xia', '0011_xisconfiguration_publisher'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='metadataledger',
            name='target_metadata_transmission_date',
        ),
        migrations.RemoveField(
            model_name='metadataledger',
            name='target_metadata_transmission_status',
        ),
        migrations.RemoveField(
            model_name='metadataledger',
            name='target_metadata_transmission_status_code',
        ),
        migrations.CreateModel(
            name='metadataTransmissionStatus',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('target_metadata_transmission_status', models.CharField(blank=True, choices=[('Successful', 'S'), ('Failed', 'F'), ('Pending', 'P'), ('Ready', 'R')], default='Ready', max_length=20)),
                ('target_metadata_transmission_status_code', models.IntegerField(blank=True, null=True)),
                ('target_metadata_transmission_date', models.DateTimeField(blank=True, null=True)),
                ('XISConfiguration', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='openlxp_xia.xisconfiguration')),
                ('metadata_record', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='openlxp_xia.metadataledger')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 3.2.18 on 2025-07-29 19:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openlxp_xia', '0010_auto_20250723_2044'),
    ]

    operations = [
        migrations.AddField(
            model_name='xisconfiguration',
            name='publisher',
            field=models.CharField(default='default', help_text='Enter the publisher name', max_length=200),
            preserve_default=False,
        ),
    ]
